==> kmplayer_0.10.3.903-1.tar.gz: kmplayer-0.10.3.903/src/kmplayer_atom.cpp <==
/**
 * Copyright (C) 2005 by Koos Vriezen <koos ! vriezen ? xs4all ! nl>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License version 2 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Steet, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 **/

//#include <config.h>
#include <string.h>
#include "kmplayer_atom.h"
#include "kmplayer_smil.h"

using namespace KMPlayer;

Node *ATOM::Feed::childFromTag (const String & tag) {
    if (!strcmp ((const char *) tag, "entry"))
        return new ATOM::Entry (m_doc);
    else if (!strcmp ((const char *) tag, "link"))
        return new ATOM::Link (m_doc);
    else if (!strcmp ((const char *) tag, "title"))
        return new DarkNode (m_doc, tag, id_node_title);
    return NULL;
}

void ATOM::Feed::closed () {
    for (Node *c = firstChild (); c; c = c->nextSibling ())
        if (c->id == id_node_title) {
            title = c->innerText ().stripWhiteSpace ();
            break;
        }
    Element::closed ();
}

void *ATOM::Feed::role (RoleType msg, void *content)
{
    if (RolePlaylist == msg)
        return !title.isEmpty () ? (PlaylistRole *) this : NULL;
    return Element::role (msg, content);
}

Node *ATOM::Entry::childFromTag (const String &tag) {
    const char *cstr = (const char *) tag;
    if (!strcmp (cstr, "link"))
        return new ATOM::Link (m_doc);
    else if (!strcmp (cstr, "content"))
        return new ATOM::Content (m_doc);
    else if (!strcmp (cstr, "title"))
        return new DarkNode (m_doc, tag, id_node_title);
    else if (!strcmp (cstr, "summary"))
        return new DarkNode (m_doc, tag, id_node_summary);
    else if (!strcmp (cstr, "media:group"))
        return new MediaGroup (m_doc);
    else if (!strcmp (cstr, "gd:rating"))
        return new DarkNode (m_doc, tag, id_node_gd_rating);
    else if (!strcmp (cstr, "category") ||
            !strcmp (cstr, "author:") ||
            !strcmp (cstr, "id") ||
            !strcmp (cstr, "updated") ||
            !strncmp (cstr, "yt:", 3) ||
            !strncmp (cstr, "gd:", 3))
        return new DarkNode (m_doc, tag, id_node_ignored);
    return NULL;
}

void ATOM::Entry::closed () {
    MediaGroup *group = NULL;
    Node *rating = NULL;
    for (Node *c = firstChild (); c; c = c->nextSibling ())
        if (c->id == id_node_title) {
            title = c->innerText ().stripWhiteSpace ();
        } else if (c->id == id_node_gd_rating) {
            rating = c;
        } else if (c->id == id_node_media_group) {
            group = static_cast <MediaGroup *> (c);
        }
    if (group)
        group->addSummary (this, rating);
    Element::closed ();
}

void *ATOM::Entry::role (RoleType msg, void *content)
{
    if (RolePlaylist == msg)
        return !title.isEmpty () ? (PlaylistRole *) this : NULL;
    return Element::role (msg, content);
}

String ATOM::Entry::description () const {
    for (Node *c = firstChild (); c; c = c->nextSibling ())
        if (c->id == id_node_content) {
            Mrl *mrl = c->mrl ();
            if (mrl->mimetype.startsWith ("text/") ||
                    mrl->mimetype.indexOf ("html") > -1)
                return mrl->innerText ();
            break;
        }
    return String ();
}

Node::PlayType ATOM::Link::playType () {
    return src.isEmpty () ? play_type_none : play_type_unknown;
}

void ATOM::Link::closed () {
    String href;
    String rel;
    for (Attribute *a = attributes ().first (); a; a = a->nextSibling ()) {
        if (a->name () == Ids::attr_href)
            href = a->value ();
        else if (a->name () == Ids::attr_title)
            title = a->value ();
        else if (a->name () == "rel")
            rel = a->value ();
    }
    if (!href.isEmpty () && rel == "enclosure")
        src = href;
    else if (title.isEmpty ())
        title = href;
    Mrl::closed ();
}

void ATOM::Content::closed () {
    for (Attribute *a = attributes ().first (); a; a = a->nextSibling ()) {
        if (a->name () == Ids::attr_src)
            src = a->value ();
        else if (a->name () == Ids::attr_type) {
            String v = a->value ().lower ();
            if (v == "text")
                mimetype = "text/plain";
            else if (v == "html")
                mimetype = "text/html";
            else if (v == "xhtml")
                mimetype = "application/xhtml+xml";
            else
                mimetype = v;
        }
    }
    Mrl::closed ();
}

Node::PlayType ATOM::Content::playType () {
    if (!hasChildNodes () && !src.isEmpty ())
        return play_type_unknown;
    return play_type_none;
}

Node *ATOM::MediaGroup::childFromTag (const String &tag) {
    const char *cstr = (const char *) tag;
    if (!strcmp (cstr, "media:content"))
        return new ATOM::MediaContent (m_doc);
    else if (!strcmp (cstr, "media:title"))
        return new DarkNode (m_doc, tag, id_node_media_title);
    else if (!strcmp (cstr, "media:description"))
        return new DarkNode (m_doc, tag, id_node_media_description);
    else if (!strcmp (cstr, "media:thumbnail"))
        return new DarkNode (m_doc, tag, id_node_media_thumbnail);
    else if (!strcmp (cstr, "media:player"))
        return new DarkNode (m_doc, tag, id_node_media_player);
    else if (!strcmp (cstr, "media:category") ||
            !strcmp (cstr, "media:keywords")||
            !strcmp (cstr, "media:credit"))
        return new DarkNode (m_doc, tag, id_node_ignored);
    else if (!strcmp (cstr, "smil"))
        return new SMIL::Smil (m_doc);
    return NULL;
}

void ATOM::MediaGroup::message (MessageType msg, void *content) {
    if (MsgChildFinished == msg &&
            id_node_media_content == ((Posting *) content)->source->id)
        finish (); // only play one
    Element::message (msg, content);
}

static String makeStar (int x, bool fill) {
    String path = "<path style=\"stroke:#A0A0A0;stroke-width:2px;stroke-opacity:1;";
    if (fill)
        path += "fill:#ff0000";
    else
        path += "fill:#C0C0C0";
    path += "\" d=\"M 21.428572,23.571429 "
        "L 10.84984,18.213257 L 0.43866021,23.890134 L 2.2655767,12.173396 "
        "L -6.3506861,4.0260275 L 5.3571425,2.142857 L 10.443179,-8.5693712 "
        "L 15.852098,1.9835038 L 27.611704,3.5103513 L 19.246772,11.915557 "
        "L 21.428572,23.571429 z\""
        " transform=\"translate(";
    path += String::number (x);
    path += ",11)\"/>";
    return path;
}

//http://code.google.com/apis/youtube/2.0/developers_guide_protocol.html
void ATOM::MediaGroup::addSummary (Node *p, Node *rating_node) {
    String images;
    String desc;
    String title;
    String player;
    String ratings;
    int img_count = 0;
    if (rating_node) {
        Element *e = static_cast <Element *> (rating_node);
        String nr = e->getAttribute ("average");
        if (!nr.isEmpty ()) {
            int rating = nr.toInt () % 6;
            ratings = "<img region=\"rating\">"
                "<svg width=\"200\" height=\"40\">";
            for (int i = 0; i < 5; ++i)
                ratings += makeStar (10 + i * 40, rating > i);
            ratings += "</svg></img>";
        }
    }
    for (Node *c = firstChild (); c; c = c->nextSibling ()) {
        switch (c->id) {
        case id_node_media_title:
            title = c->innerText ();
            break;
        case id_node_media_description:
            desc = c->innerText ();
            break;
        case id_node_media_player:
            player = static_cast <Element *> (c)->getAttribute (Ids::attr_url);
            break;
        case id_node_media_thumbnail:
        {
            Element *e = static_cast <Element *> (c);
            String url = e->getAttribute (Ids::attr_url);
            if (!url.isEmpty ()) {
                images += String ("<img region=\"image\" src=\"") + url + Char ('"');
                String w = e->getAttribute (Ids::attr_width);
                if (!w.isEmpty ())
                    images += String (" width=\"") + w + Char ('"');
                String h = e->getAttribute (Ids::attr_height);
                if (!h.isEmpty ())
                    images += String (" height=\"") + h + Char ('"');
                images += String (" dur=\"20\" transIn=\"fade\" fill=\"transition\" fit=\"meet\"/>");
                img_count++;
            }
            break;
        }
        }
    }
    if (img_count) {
        TextStream out;
        out << "<smil><head>";
        if (!title.isEmpty ())
            out << "<title>" << title << "</title>";
        out << "<layout><root-layout width=\"400\" height=\"300\" background-color=\"#F5F5DC\"/>";
        if (!title.isEmpty ())
            out << "<region id=\"title\" left=\"20\" top=\"10\" height=\"18\" right=\"10\"/>";
        out << "<region id=\"image\" left=\"5\" top=\"40\" width=\"130\" bottom=\"30\"/>";
        if (!ratings.isEmpty ())
            out << "<region id=\"rating\" left=\"15\" width=\"100\" height=\"20\" bottom=\"5\"/>";
        out << "<region id=\"text\" left=\"140\" top=\"40\" bottom=\"10\" right=\"10\" fit=\"scroll\"/>"
            "</layout>"
            "<transition id=\"fade\" dur=\"0.3\" type=\"fade\"/>"
            "</head><body>"
            "<par><seq repeatCount=\"indefinite\">";
        out << images;
        out << "</seq>";
        if (!title.isEmpty ()) {
            if (!player.isEmpty ())
                out << "<a href=\"" << XMLStringlet(player) << "\" target=\"top\">";
            out << "<smilText region=\"title\" textFontWeight=\"bold\" textFontSize=\"12\"";
            if (!player.isEmpty ())
                out << " textColor=\"blue\"";
            out << ">" << XMLStringlet (title) << "</smilText>";
            if (!player.isEmpty ())
                out << "</a>";
        }
        if (!ratings.isEmpty ())
            out << ratings;
        out << "<smilText region=\"text\" textFontSize=\"12\">";
        out << XMLStringlet (desc);
        out << String ("</smilText></par></body></smil>");
        insertBefore (out.release (), firstChild ());
        if (firstChild () && SMIL::id_node_smil == firstChild ()->id) {
            NodePtr n = firstChild ();
            n->auxiliary_node = true;
            removeChild (n);
            p->insertBefore (n, p->firstChild ());
        }
    }
}

void ATOM::MediaContent::closed () {
    int fsize = 0;
    TrieString fs ("fileSize");
    for (Attribute *a = attributes ().first (); a; a = a->nextSibling ()) {
        if (a->name () == Ids::attr_url)
            src = a->value();
        else if (a->name () == Ids::attr_type)
            mimetype = a->value ();
        else if (a->name () == Ids::attr_height)
            size.height = a->value ().toInt ();
        else if (a->name () == Ids::attr_width)
            size.width = a->value ().toInt ();
        else if (a->name () == Ids::attr_width)
            size.width = a->value ().toInt ();
        else if (a->name () == fs)
            fsize = a->value ().toInt ();
    }
    if (!mimetype.isEmpty ()) {
        title = mimetype;
        if (fsize > 0) {
            if (fsize > 1024 * 1024)
                title += String (" (") + String::number (fsize / (1024 * 1024)) + String (" Mb)");
            else
                title += String (" (") + String::number (fsize / 1024) + String (" kb)");
        }
    }
    Mrl::closed ();
}

==> kmplayer_0.10.4-1.tar.gz: kmplayer-0.10.4/src/kmplayer_atom.cpp <==
/**
 * Copyright (C) 2005 by Koos Vriezen <koos ! vriezen ? xs4all ! nl>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License version 2 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Steet, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 **/

//#include <config.h>
#include <string.h>
#include "kmplayer_atom.h"
#include "kmplayer_smil.h"

using namespace KMPlayer;

Node *ATOM::Feed::childFromTag (const String & tag) {
    if (!strcmp ((const char *) tag, "entry"))
        return new ATOM::Entry (m_doc);
    else if (!strcmp ((const char *) tag, "link"))
        return new ATOM::Link (m_doc);
    else if (!strcmp ((const char *) tag, "title"))
        return new DarkNode (m_doc, tag, id_node_title);
    return NULL;
}

void ATOM::Feed::closed () {
    for (Node *c = firstChild (); c; c = c->nextSibling ())
        if (c->id == id_node_title) {
            title = c->innerText ().stripWhiteSpace ();
            break;
        }
    Element::closed ();
}

void *ATOM::Feed::role (RoleType msg, void *content)
{
    if (RolePlaylist == msg)
        return !title.isEmpty () ? (PlaylistRole *) this : NULL;
    return Element::role (msg, content);
}

Node *ATOM::Entry::childFromTag (const String &tag) {
    const char *cstr = (const char *) tag;
    if (!strcmp (cstr, "link"))
        return new ATOM::Link (m_doc);
    else if (!strcmp (cstr, "content"))
        return new ATOM::Content (m_doc);
    else if (!strcmp (cstr, "title"))
        return new DarkNode (m_doc, tag, id_node_title);
    else if (!strcmp (cstr, "summary"))
        return new DarkNode (m_doc, tag, id_node_summary);
    else if (!strcmp (cstr, "media:group"))
        return new MediaGroup (m_doc);
    else if (!strcmp (cstr, "gd:rating") ||
            !strcmp (cstr, "category") ||
            !strcmp (cstr, "author:") ||
            !strcmp (cstr, "id") ||
            !strcmp (cstr, "updated") ||
            !strncmp (cstr, "yt:", 3) ||
            !strncmp (cstr, "gd:", 3))
        return new DarkNode (m_doc, tag, id_node_ignored);
    return NULL;
}

void ATOM::Entry::closed () {
    MediaGroup *group = NULL;
    for (Node *c = firstChild (); c; c = c->nextSibling ())
        if (c->id == id_node_title) {
            title = c->innerText ().stripWhiteSpace ();
        } else if (c->id == id_node_media_group) {
            group = static_cast <MediaGroup *> (c);
        }
    Element::closed ();
}

void *ATOM::Entry::role (RoleType msg, void *content)
{
    if (RolePlaylist == msg)
        return !title.isEmpty () ? (PlaylistRole *) this : NULL;
    return Element::role (msg, content);
}

String ATOM::Entry::description () const {
    for (Node *c = firstChild (); c; c = c->nextSibling ())
        if (c->id == id_node_content) {
            Mrl *mrl = c->mrl ();
            if (mrl->mimetype.startsWith ("text/") ||
                    mrl->mimetype.indexOf ("html") > -1)
                return mrl->innerText ();
            break;
        }
    return String ();
}

Node::PlayType ATOM::Link::playType () {
    return src.isEmpty () ? play_type_none : play_type_unknown;
}

void ATOM::Link::closed () {
    String href;
    String rel;
    for (Attribute *a = attributes ().first (); a; a = a->nextSibling ()) {
        if (a->name () == Ids::attr_href)
            href = a->value ();
        else if (a->name () == Ids::attr_title)
            title = a->value ();
        else if (a->name () == "rel")
            rel = a->value ();
    }
    if (!href.isEmpty () && rel == "enclosure")
        src = href;
    else if (title.isEmpty ())
        title = href;
    Mrl::closed ();
}

void ATOM::Content::closed () {
    for (Attribute *a = attributes ().first (); a; a = a->nextSibling ()) {
        if (a->name () == Ids::attr_src)
            src = a->value ();
        else if (a->name () == Ids::attr_type) {
            String v = a->value ().lower ();
            if (v == "text")
                mimetype = "text/plain";
            else if (v == "html")
                mimetype = "text/html";
            else if (v == "xhtml")
                mimetype = "application/xhtml+xml";
            else
                mimetype = v;
        }
    }
    Mrl::closed ();
}

Node::PlayType ATOM::Content::playType () {
    if (!hasChildNodes () && !src.isEmpty ())
        return play_type_unknown;
    return play_type_none;
}

Node *ATOM::MediaGroup::childFromTag (const String &tag) {
    const char *cstr = (const char *) tag;
    if (!strcmp (cstr, "media:content"))
        return new ATOM::MediaContent (m_doc);
    else if (!strcmp (cstr, "media:title"))
        return new DarkNode (m_doc, tag, id_node_media_title);
    else if (!strcmp (cstr, "media:description"))
        return new DarkNode (m_doc, tag, id_node_media_description);
    else if (!strncmp (cstr, "media:", 6))
        return new DarkNode (m_doc, tag, id_node_ignored);
    else if (!strcmp (cstr, "smil"))
        return new SMIL::Smil (m_doc);
    return NULL;
}

void ATOM::MediaGroup::closed () {
    String title;
    for (Node *c = firstChild (); c; c = c->nextSibling ())
        if (id_node_media_title == c->id) {
            title = c->innerText ();
            break;
        }
    if (!title.isEmpty ())
        for (Node *c = firstChild (); c; c = c->nextSibling ())
            if (id_node_media_content == c->id) {
                Mrl *mrl = c->mrl ();
                if (mrl && !mrl->src.isEmpty ()) {
                    String s = title;
                    if (!mrl->mimetype.isEmpty ())
                        s += Char (' ') + c->mrl ()->mimetype;
                    c->mrl ()->setCaption (s);
                }
            }
}

void ATOM::MediaGroup::message (MessageType msg, void *content) {
    if (MsgChildFinished == msg &&
            id_node_media_content == ((Posting *) content)->source->id)
        finish (); // only play one
    Element::message (msg, content);
}

void ATOM::MediaContent::closed () {
    for (Attribute *a = attributes ().first (); a; a = a->nextSibling ()) {
        if (a->name () == Ids::attr_url)
            src = a->value();
        else if (a->name () == Ids::attr_type)
            mimetype = a->value ();
        else if (a->name () == Ids::attr_height)
            size.height = a->value ().toInt ();
        else if (a->name () == Ids::attr_width)
            size.width = a->value ().toInt ();
        else if (a->name () == Ids::attr_width)
            size.width = a->value ().toInt ();
    }
    Mrl::closed ();
}
